@@ -36,12 +36,6 @@
 #include "config_options.h"
 #include "util/enum_iterator.h"
 
-class AutoscanDirectory;
-class AutoscanList;
-enum class AutoscanScanMode;
-class DirectoryTweak;
-class DynamicContent;
-
 #define YES "yes"
 #define NO "no"
 
@@ -55,6 +49,7 @@ using ArrayItemCheckFunction = std::function<bool(const std::string& value)>;
 using DictionaryInitFunction = std::function<bool(const pugi::xml_node& value, std::map<std::string, std::string>& result)>;
 using IntCheckFunction = std::function<bool(int value)>;
 using IntParseFunction = std::function<int(const std::string& value)>;
+using IntPrintFunction = std::function<std::string(int value)>;
 using IntMinFunction = std::function<bool(int value, int minValue)>;
 
 using ConfigOptionIterator = EnumIterator<config_option_t, config_option_t::CFG_MIN, config_option_t::CFG_MAX>;
@@ -200,86 +195,31 @@ public:
 
     std::shared_ptr<ConfigOption> newOption(const std::string& optValue);
 
-    static bool CheckSqlJournalMode(std::string& value);
+    static bool CheckCharset(std::string& value);
 };
 
-template <class En>
-class ConfigEnumSetup : public ConfigSetup {
-protected:
-    bool notEmpty = true;
-    std::map<std::string, En> valueMap;
-
-public:
-    ConfigEnumSetup(config_option_t option, const char* xpath, const char* help, std::map<std::string, En> valueMap, bool notEmpty = false)
-        : ConfigSetup(option, xpath, help, false, "")
-        , notEmpty(notEmpty)
-        , valueMap(std::move(valueMap))
-    {
-    }
-
-    ConfigEnumSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue, std::map<std::string, En> valueMap, bool notEmpty = false)
-        : ConfigSetup(option, xpath, help, false, defaultValue)
-        , notEmpty(notEmpty)
-        , valueMap(std::move(valueMap))
-    {
-    }
-
-    std::string getTypeString() const override { return "Enum"; }
-
-    void makeOption(const pugi::xml_node& root, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override
-    {
-        if (arguments && arguments->find("notEmpty") != arguments->end()) {
-            notEmpty = arguments->find("notEmpty")->second == "true";
-        }
-        newOption(ConfigSetup::getXmlContent(root, true));
-        setOption(config);
-    }
-
-    bool checkEnumValue(const std::string& value, En& result) const
-    {
-        if (valueMap.find(value) != valueMap.end()) {
-            result = valueMap.at(value);
-            return true;
-        }
-        return false;
-    }
-
-    En getXmlContent(const pugi::xml_node& root)
-    {
-        std::string optValue = ConfigSetup::getXmlContent(root, true);
-        log_debug("Config: option: '{}' value: '{}'", xpath, optValue);
-        if (notEmpty && optValue.empty()) {
-            throw_std_runtime_error("Error in config file: Invalid {}/{} empty value '{}'", root.path(), xpath, optValue);
-        }
-        En result;
-        if (!checkEnumValue(optValue, result)) {
-            throw_std_runtime_error("Error in config file: {}/{} unsupported Enum value '{}'", root.path(), xpath, optValue);
-        }
-        return result;
-    }
-
-    std::shared_ptr<ConfigOption> newOption(const std::string& optValue)
-    {
-        if (notEmpty && optValue.empty()) {
-            throw_std_runtime_error("Invalid {} empty value '{}'", xpath, optValue);
-        }
-        En result;
-        if (!checkEnumValue(optValue, result)) {
-            throw_std_runtime_error("Error in config file: {} unsupported Enum value '{}'", xpath, optValue);
-        }
-        optionValue = std::make_shared<Option>(optValue);
-        return optionValue;
-    }
+enum class ConfigPathArguments {
+    none = 0,
+    isFile = (1 << 0),
+    mustExist = (1 << 1),
+    notEmpty = (1 << 2),
+    isExe = (1 << 3),
+    resolveEmpty = (1 << 4),
 };
 
+inline ConfigPathArguments operator|(ConfigPathArguments a, ConfigPathArguments b)
+{
+    return static_cast<ConfigPathArguments>(static_cast<int>(a) | static_cast<int>(b));
+}
+
+inline ConfigPathArguments operator&(ConfigPathArguments a, ConfigPathArguments b)
+{
+    return static_cast<ConfigPathArguments>(static_cast<int>(a) & static_cast<int>(b));
+}
+
 class ConfigPathSetup : public ConfigSetup {
 protected:
-    bool isFile = false;
-    bool mustExist = false;
-    bool notEmpty = false;
-    bool isExe = false;
-    bool resolveEmpty = true;
-
+    ConfigPathArguments arguments;
     /// \brief resolve path against home, an exception is raised if path does not exist on filesystem.
     /// \param path path to be resolved
     /// \param isFile file or directory
@@ -289,15 +229,14 @@ protected:
     void loadArguments(const std::map<std::string, std::string>* arguments = nullptr);
     bool checkExecutable(std::string& optValue) const;
 
+    bool isSet(ConfigPathArguments a) const { return (arguments & a) == a; }
+
 public:
     static fs::path Home;
 
-    ConfigPathSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue = "", bool isFile = false, bool mustExist = true, bool notEmpty = false, bool isExe = false)
+    ConfigPathSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue = "", ConfigPathArguments arguments = ConfigPathArguments::mustExist | ConfigPathArguments::resolveEmpty)
         : ConfigSetup(option, xpath, help, false, defaultValue)
-        , isFile(isFile)
-        , mustExist(mustExist)
-        , notEmpty(notEmpty)
-        , isExe(isExe)
+        , arguments(ConfigPathArguments(arguments))
     {
     }
 
@@ -312,27 +251,26 @@ public:
 
     bool checkPathValue(std::string& optValue, std::string& pathValue) const;
 
-    void setMustExist(bool mustExist) { this->mustExist = mustExist; }
+    void setFlag(bool hasFlag, ConfigPathArguments flag);
 };
 
 class ConfigIntSetup : public ConfigSetup {
 protected:
     IntCheckFunction valueCheck = nullptr;
     IntParseFunction parseValue = nullptr;
+    IntPrintFunction printValue = nullptr;
     IntMinFunction minCheck = nullptr;
     int minValue {};
 
 public:
     ConfigIntSetup(config_option_t option, const char* xpath, const char* help)
         : ConfigSetup(option, xpath, help)
-
     {
         this->defaultValue = fmt::to_string(0);
     }
 
     ConfigIntSetup(config_option_t option, const char* xpath, const char* help, int defaultValue)
         : ConfigSetup(option, xpath, help)
-
     {
         this->defaultValue = fmt::to_string(defaultValue);
     }
@@ -340,23 +278,22 @@ public:
     ConfigIntSetup(config_option_t option, const char* xpath, const char* help, IntCheckFunction check)
         : ConfigSetup(option, xpath, help)
         , valueCheck(std::move(check))
-
     {
         this->defaultValue = fmt::to_string(0);
     }
 
-    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, int defaultValue, IntParseFunction parseValue)
+    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, int defaultValue, IntParseFunction parseValue, IntPrintFunction printValue = nullptr)
         : ConfigSetup(option, xpath, help)
         , parseValue(std::move(parseValue))
-
+        , printValue(std::move(printValue))
     {
         this->defaultValue = fmt::to_string(defaultValue);
     }
 
-    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, int defaultValue, IntCheckFunction check)
+    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, int defaultValue, IntCheckFunction check, IntPrintFunction printValue = nullptr)
         : ConfigSetup(option, xpath, help)
         , valueCheck(std::move(check))
-
+        , printValue(std::move(printValue))
     {
         this->defaultValue = fmt::to_string(defaultValue);
     }
@@ -369,17 +306,17 @@ public:
         this->defaultValue = fmt::to_string(defaultValue);
     }
 
-    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue, IntCheckFunction check = nullptr)
+    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue, IntCheckFunction check = nullptr, IntPrintFunction printValue = nullptr)
         : ConfigSetup(option, xpath, help)
         , valueCheck(std::move(check))
-
+        , printValue(std::move(printValue))
     {
         this->defaultValue = defaultValue;
     }
 
-    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue, StringCheckFunction check)
+    ConfigIntSetup(config_option_t option, const char* xpath, const char* help, const char* defaultValue, StringCheckFunction check, IntPrintFunction printValue = nullptr)
         : ConfigSetup(option, xpath, help, std::move(check), defaultValue)
-
+        , printValue(std::move(printValue))
     {
     }
 
@@ -395,9 +332,7 @@ public:
 
     int checkIntValue(std::string& sVal, const std::string& pathName = "") const;
 
-    std::string getCurrentValue() const override { return optionValue ? fmt::to_string(optionValue->getIntOption()) : ""; }
-
-    static bool CheckSqlLiteSyncValue(std::string& value);
+    std::string getCurrentValue() const override { return optionValue ? optionValue->getOption() : ""; }
 
     static bool CheckProfileNumberValue(std::string& value);
 
@@ -445,8 +380,6 @@ public:
 
     std::string getCurrentValue() const override { return optionValue ? fmt::to_string(optionValue->getBoolOption()) : ""; }
 
-    static bool CheckSqlLiteRestoreValue(std::string& value);
-
     static bool CheckInotifyValue(std::string& value);
 
     static bool CheckMarkPlayedValue(std::string& value);
@@ -658,138 +591,4 @@ public:
     std::string getCurrentValue() const override { return {}; }
 };
 
-class ConfigAutoscanSetup : public ConfigSetup {
-protected:
-    AutoscanScanMode scanMode;
-    bool hiddenFiles = false;
-
-    /// \brief Creates an array of AutoscanDirectory objects from a XML nodeset.
-    /// \param element starting element of the nodeset.
-    /// \param scanmode add only directories with the specified scanmode to the array
-    bool createOptionFromNode(const pugi::xml_node& element, std::vector<AutoscanDirectory>& result);
-
-    bool updateItem(std::size_t i, const std::string& optItem, const std::shared_ptr<Config>& config, AutoscanDirectory& entry, std::string& optValue, const std::string& status = "") const;
-
-public:
-    ConfigAutoscanSetup(config_option_t option, const char* xpath, const char* help, AutoscanScanMode scanmode)
-        : ConfigSetup(option, xpath, help)
-        , scanMode(scanmode)
-    {
-    }
-
-    std::string getTypeString() const override { return "List"; }
-
-    void makeOption(const pugi::xml_node& root, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    bool updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    std::string getUniquePath() const override;
-
-    std::string getItemPath(int index = 0, config_option_t propOption = CFG_MAX, config_option_t propOption2 = CFG_MAX, config_option_t propOption3 = CFG_MAX, config_option_t propOption4 = CFG_MAX) const override;
-
-    std::shared_ptr<ConfigOption> newOption(const pugi::xml_node& optValue);
-
-    std::string getCurrentValue() const override { return {}; }
-};
-
-class ConfigTranscodingSetup : public ConfigSetup {
-    using ConfigSetup::ConfigSetup;
-
-protected:
-    bool isEnabled = false;
-
-    /// \brief Creates an array of TranscodingProfile objects from an XML
-    /// nodeset.
-    /// \param element starting element of the nodeset.
-    static bool createOptionFromNode(const pugi::xml_node& element, std::shared_ptr<TranscodingProfileList>& result);
-
-public:
-    std::string getTypeString() const override { return "List"; }
-
-    void makeOption(const pugi::xml_node& root, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    bool updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    std::string getItemPath(int index = 0, config_option_t propOption = CFG_MAX, config_option_t propOption2 = CFG_MAX, config_option_t propOption3 = CFG_MAX, config_option_t propOption4 = CFG_MAX) const override;
-
-    std::shared_ptr<ConfigOption> newOption(const pugi::xml_node& optValue);
-
-    std::string getCurrentValue() const override { return {}; }
-};
-
-class ConfigClientSetup : public ConfigSetup {
-    using ConfigSetup::ConfigSetup;
-
-protected:
-    bool isEnabled = false;
-
-    /// \brief Creates an array of ClientConfig objects from a XML nodeset.
-    /// \param element starting element of the nodeset.
-    static bool createOptionFromNode(const pugi::xml_node& element, const std::shared_ptr<ClientConfigList>& result);
-
-    bool updateItem(std::size_t i, const std::string& optItem, const std::shared_ptr<Config>& config, std::shared_ptr<ClientConfig>& entry, std::string& optValue, const std::string& status = "") const;
-
-public:
-    std::string getTypeString() const override { return "List"; }
-
-    void makeOption(const pugi::xml_node& root, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    bool updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    std::string getItemPath(int index = 0, config_option_t propOption = CFG_MAX, config_option_t propOption2 = CFG_MAX, config_option_t propOption3 = CFG_MAX, config_option_t propOption4 = CFG_MAX) const override;
-
-    std::shared_ptr<ConfigOption> newOption(const pugi::xml_node& optValue);
-
-    std::string getCurrentValue() const override { return {}; }
-};
-
-class ConfigDirectorySetup : public ConfigSetup {
-    using ConfigSetup::ConfigSetup;
-
-protected:
-    /// \brief Creates an array of DirectoryTweak objects from a XML nodeset.
-    /// \param element starting element of the nodeset.
-    static bool createOptionFromNode(const pugi::xml_node& element, std::shared_ptr<DirectoryConfigList>& result);
-
-    bool updateItem(std::size_t i, const std::string& optItem, const std::shared_ptr<Config>& config, std::shared_ptr<DirectoryTweak>& entry, std::string& optValue, const std::string& status = "") const;
-
-public:
-    std::string getTypeString() const override { return "List"; }
-
-    void makeOption(const pugi::xml_node& root, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    bool updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    std::string getItemPath(int index = 0, config_option_t propOption = CFG_MAX, config_option_t propOption2 = CFG_MAX, config_option_t propOption3 = CFG_MAX, config_option_t propOption4 = CFG_MAX) const override;
-
-    std::shared_ptr<ConfigOption> newOption(const pugi::xml_node& optValue);
-
-    std::string getCurrentValue() const override { return {}; }
-};
-
-/// \brief Setup of dynamic content reader
-class ConfigDynamicContentSetup : public ConfigSetup {
-    using ConfigSetup::ConfigSetup;
-
-protected:
-    /// \brief Creates an array of DynamicContent objects from a XML nodeset.
-    /// \param element starting element of the nodeset.
-    static bool createOptionFromNode(const pugi::xml_node& element, std::shared_ptr<DynamicContentList>& result);
-
-    bool updateItem(std::size_t i, const std::string& optItem, const std::shared_ptr<Config>& config, std::shared_ptr<DynamicContent>& entry, std::string& optValue, const std::string& status = "") const;
-
-public:
-    std::string getTypeString() const override { return "List"; }
-
-    void makeOption(const pugi::xml_node& root, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    bool updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments = nullptr) override;
-
-    std::string getItemPath(int index = 0, config_option_t propOption = CFG_MAX, config_option_t propOption2 = CFG_MAX, config_option_t propOption3 = CFG_MAX, config_option_t propOption4 = CFG_MAX) const override;
-
-    std::shared_ptr<ConfigOption> newOption(const pugi::xml_node& optValue);
-
-    std::string getCurrentValue() const override { return {}; }
-};
-
 #endif // __CONFIG_SETUP_H__
