@@ -22,14 +22,15 @@
 
 /// \file config_setup_transcoding.cc
 
-#include "config/config_setup.h" // API
+#include "config/setup/config_setup_transcoding.h" // API
 
 #include <numeric>
 
 #include "config/client_config.h"
 #include "config/config_definition.h"
+#include "config/config_option_enum.h"
 #include "config/config_options.h"
-#include "content/autoscan.h"
+#include "config/setup/config_setup_enum.h"
 #include "metadata/metadata_handler.h"
 #include "metadata/resolution.h"
 #include "transcoding/transcoding.h"
@@ -65,6 +66,11 @@ bool ConfigTranscodingSetup::createOptionFromNode(const pugi::xml_node& element,
             ConfigDefinition::findConfigSetup<ConfigStringSetup>(ATTR_TRANSCODING_MIMETYPE_PROF_MAP_USING)->getXmlContent(child));
         filter->setSourceProfile(ConfigDefinition::findConfigSetup<ConfigStringSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_SRCDLNA)->getXmlContent(child));
         filter->setClientFlags(ConfigDefinition::findConfigSetup<ConfigIntSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_CLIENTFLAGS)->getXmlContent(child));
+        auto noTranscoding = ConfigDefinition::findConfigSetup<ConfigStringSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_NOTRANSCODING)->getXmlContent(child);
+        std::vector<std::string> noTranscodingVector;
+        for (auto&& mime : splitString(noTranscoding, ','))
+            noTranscodingVector.push_back(trimString(mime));
+        filter->setNoTranscodingMimeTypes(noTranscodingVector);
         trFilters.push_back(filter);
         result->add(filter);
     }
@@ -82,7 +88,7 @@ bool ConfigTranscodingSetup::createOptionFromNode(const pugi::xml_node& element,
 
         auto prof = std::make_shared<TranscodingProfile>(
             ConfigDefinition::findConfigSetup<ConfigBoolSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_ENABLED)->getXmlContent(child),
-            ConfigDefinition::findConfigSetup<ConfigEnumSetup<transcoding_type_t>>(ATTR_TRANSCODING_PROFILES_PROFLE_TYPE)->getXmlContent(child),
+            ConfigDefinition::findConfigSetup<ConfigEnumSetup<TranscodingType>>(ATTR_TRANSCODING_PROFILES_PROFLE_TYPE)->getXmlContent(child),
             ConfigDefinition::findConfigSetup<ConfigStringSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_NAME)->getXmlContent(child));
         prof->setTargetMimeType(ConfigDefinition::findConfigSetup<ConfigStringSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_MIMETYPE)->getXmlContent(child));
         prof->setClientFlags(ConfigDefinition::findConfigSetup<ConfigIntSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_CLIENTFLAGS)->getXmlContent(child));
@@ -106,8 +112,8 @@ bool ConfigTranscodingSetup::createOptionFromNode(const pugi::xml_node& element,
             auto cs = ConfigDefinition::findConfigSetup<ConfigArraySetup>(ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC);
             if (cs->hasXmlElement(child)) {
                 sub = cs->getXmlElement(child);
-                avi_fourcc_listmode_t fccMode = ConfigDefinition::findConfigSetup<ConfigEnumSetup<avi_fourcc_listmode_t>>(ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE)->getXmlContent(sub);
-                if (fccMode != FCC_None) {
+                AviFourccListmode fccMode = ConfigDefinition::findConfigSetup<ConfigEnumSetup<AviFourccListmode>>(ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE)->getXmlContent(sub);
+                if (fccMode != AviFourccListmode::None) {
                     prof->setAVIFourCCList(cs->getXmlContent(sub), fccMode);
                 }
             }
@@ -158,7 +164,7 @@ bool ConfigTranscodingSetup::createOptionFromNode(const pugi::xml_node& element,
         sub = ConfigDefinition::findConfigSetup<ConfigSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_AGENT)->getXmlElement(child);
         {
             auto cs = ConfigDefinition::findConfigSetup<ConfigPathSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_AGENT_COMMAND);
-            cs->setMustExist(prof->isEnabled());
+            cs->setFlag(prof->isEnabled(), ConfigPathArguments::mustExist);
             prof->setCommand(cs->getXmlContent(sub));
         }
         prof->setArguments(ConfigDefinition::findConfigSetup<ConfigStringSetup>(ATTR_TRANSCODING_PROFILES_PROFLE_AGENT_ARGS)->getXmlContent(sub));
@@ -288,6 +294,16 @@ bool ConfigTranscodingSetup::updateDetail(const std::string& optItem, std::strin
                 log_debug("New Transcoding Detail {} {}", index, filter->getSourceProfile());
                 return true;
             }
+            index = getItemPath(i, ATTR_TRANSCODING_MIMETYPE_FILTER, ATTR_TRANSCODING_PROFILES_PROFLE_NOTRANSCODING);
+            if (optItem == index) {
+                log_error("Cannot change profile name in Transcoding Detail {} {}", index, fmt::join(filter->getNoTranscodingMimeTypes(), ","));
+                std::vector<std::string> noTranscodingVector;
+                for (auto&& mime : splitString(optValue, ','))
+                    noTranscodingVector.push_back(trimString(mime));
+                filter->setNoTranscodingMimeTypes(noTranscodingVector);
+                log_debug("New Transcoding Detail {} {}", index, fmt::join(filter->getNoTranscodingMimeTypes(), ","));
+                return true;
+            }
 
             if (filter->getTranscodingProfile())
                 profiles[filter->getTranscodingProfile()->getName()] = filter->getTranscodingProfile();
@@ -318,9 +334,10 @@ bool ConfigTranscodingSetup::updateDetail(const std::string& optItem, std::strin
             }
             index = getItemPath(i, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_TYPE);
             if (optItem == index) {
-                transcoding_type_t type;
-                if (ConfigDefinition::findConfigSetup<ConfigEnumSetup<transcoding_type_t>>(ATTR_TRANSCODING_PROFILES_PROFLE_TYPE)->checkEnumValue(optValue, type)) {
-                    config->setOrigValue(index, entry->getType());
+                TranscodingType type;
+                auto setup = ConfigDefinition::findConfigSetup<ConfigEnumSetup<TranscodingType>>(ATTR_TRANSCODING_PROFILES_PROFLE_TYPE);
+                if (setup->checkEnumValue(optValue, type)) {
+                    config->setOrigValue(index, setup->mapEnumValue(entry->getType()));
                     entry->setType(type);
                     log_debug("New Transcoding Detail {} {}", index, config->getTranscodingProfileListOption(option)->getByName(entry->getName(), true)->getType());
                     return true;
@@ -464,13 +481,14 @@ bool ConfigTranscodingSetup::updateDetail(const std::string& optItem, std::strin
             }
 
             // update 4cc options
-            avi_fourcc_listmode_t fccMode = entry->getAVIFourCCListMode();
+            AviFourccListmode fccMode = entry->getAVIFourCCListMode();
             auto fccList = entry->getAVIFourCCList();
             bool set4cc = false;
             index = getItemPath(i, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE);
             if (optItem == index) {
-                config->setOrigValue(index, TranscodingProfile::mapFourCcMode(fccMode));
-                if (ConfigDefinition::findConfigSetup<ConfigEnumSetup<avi_fourcc_listmode_t>>(ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE)->checkEnumValue(optValue, fccMode)) {
+                auto setup = ConfigDefinition::findConfigSetup<ConfigEnumSetup<AviFourccListmode>>(ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE);
+                config->setOrigValue(index, setup->mapEnumValue(fccMode));
+                if (setup->checkEnumValue(optValue, fccMode)) {
                     set4cc = true;
                 }
             }
@@ -497,7 +515,7 @@ std::shared_ptr<ConfigOption> ConfigTranscodingSetup::newOption(const pugi::xml_
     auto result = std::make_shared<TranscodingProfileList>();
 
     if (!createOptionFromNode(isEnabled ? optValue : pugi::xml_node(nullptr), result)) {
-        throw_std_runtime_error("Init {} transcoding failed '{}'", xpath, optValue);
+        throw_std_runtime_error("Init {} transcoding failed '{}'", xpath, optValue.name());
     }
     optionValue = std::make_shared<TranscodingProfileListOption>(result);
     return optionValue;
