@@ -35,8 +35,6 @@
 #ifdef HAVE_INOTIFY
 #include "autoscan_inotify.h" // API
 
-#include <sstream>
-
 #include "content_manager.h"
 #include "database/database.h"
 #include "util/tools.h"
@@ -48,6 +46,8 @@ AutoscanInotify::AutoscanInotify(const std::shared_ptr<ContentManager>& content)
     , database(content->getContext()->getDatabase())
     , content(content)
 {
+    defFollowSymlinks = this->config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS);
+    defHidden = this->config->getBoolOption(CFG_IMPORT_HIDDEN_FILES);
     std::error_code ec;
     if (isRegularFile(INOTIFY_MAX_USER_WATCHES_FILE, ec)) {
         try {
@@ -100,9 +100,14 @@ void AutoscanInotify::threadProc()
             while (!unmonitorQueue.empty()) {
                 auto adir = std::move(unmonitorQueue.front());
                 unmonitorQueue.pop();
-
                 lock.unlock();
 
+                if (!adir) {
+                    log_debug("Empty autoscan");
+                    lock.lock();
+                    continue;
+                }
+
                 const fs::path& location = adir->getLocation();
                 if (location.empty()) {
                     lock.lock();
@@ -113,7 +118,7 @@ void AutoscanInotify::threadProc()
 
                 if (adir->getRecursive()) {
                     log_debug("Removing recursive watch: {}", location.c_str());
-                    monitorUnmonitorRecursive(dirEnt, true, adir, true, config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS));
+                    monitorUnmonitorRecursive(dirEnt, true, adir, true, adir->getFollowSymlinks());
                 } else {
                     log_debug("Removing non-recursive watch: {}", location.c_str());
                     unmonitorDirectory(location, adir);
@@ -130,6 +135,7 @@ void AutoscanInotify::threadProc()
                 lock.unlock();
 
                 if (!adir) {
+                    log_debug("Empty autoscan");
                     lock.lock();
                     continue;
                 }
@@ -146,7 +152,7 @@ void AutoscanInotify::threadProc()
                     // handle dir recursively
                     if (adir->getRecursive()) {
                         log_debug("Adding recursive watch: {}", location.c_str());
-                        monitorUnmonitorRecursive(dirEnt, false, adir, true, config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS));
+                        monitorUnmonitorRecursive(dirEnt, false, adir, true, adir->getFollowSymlinks());
                     } else {
                         log_debug("Adding non-recursive watch: {}", location.c_str());
                         monitorDirectory(location, adir, true);
@@ -166,28 +172,43 @@ void AutoscanInotify::threadProc()
             /* --- */
 
             if (event) {
+                // lock.lock();
                 int wd = event->wd;
-                int mask = event->mask;
-                std::string name = event->name;
-                log_debug("inotify event: {} 0x{:x} {}", wd, mask, name);
+                uint32_t mask = event->mask;
+                std::string name = event->len > 0 ? event->name : "";
+                log_debug("inotify event: {} mask=0x{:x} name={}", wd, mask, name);
 
                 std::shared_ptr<Wd> wdObj;
                 try {
                     wdObj = watches.at(wd);
                 } catch (const std::out_of_range&) {
                     inotify->removeWatch(wd);
+                    // lock.unlock();
                     continue;
                 }
 
                 fs::path path = wdObj->getPath();
                 // file is not gone
-                if (!(mask & (IN_DELETE_SELF | IN_MOVE_SELF | IN_UNMOUNT)))
+                if (mask & IN_MOVE_SELF)
+                    path = path.parent_path() / name;
+                else if (!(mask & (IN_DELETE_SELF | IN_MOVE_SELF | IN_UNMOUNT)) && !name.empty())
                     path /= name;
 
+                auto dirEnt = fs::directory_entry(path, ec);
+                bool isDir = mask & IN_ISDIR;
+                if (!ec) {
+                    isDir = isDir || dirEnt.is_directory(ec);
+                }
+                if (ec && !(mask & (IN_DELETE_SELF | IN_DELETE | IN_MOVED_FROM | IN_MOVE_SELF | IN_UNMOUNT))) {
+                    log_error("Failed to read {}: {}", path.c_str(), ec.message());
+                }
                 std::shared_ptr<AutoscanDirectory> adir;
                 auto watchAs = getAppropriateAutoscan(wdObj, path);
                 if (watchAs)
                     adir = watchAs->getAutoscanDirectory();
+                if (!watchAs || !adir) {
+                    log_debug("autoscan not found in watches? ({}, watchAs:{}, adir:{}, {})", wd, watchAs == nullptr, adir == nullptr, path.c_str());
+                }
 
                 // file is renamed
                 if (mask & IN_MOVE_SELF) {
@@ -198,24 +219,27 @@ void AutoscanInotify::threadProc()
                 if (mask & (IN_DELETE_SELF | IN_MOVE_SELF | IN_UNMOUNT)) {
                     recheckNonexistingMonitors(wd, wdObj);
                 }
+                AutoScanSetting asSetting;
+                asSetting.adir = adir;
+                asSetting.followSymlinks = adir ? adir->getFollowSymlinks() : defFollowSymlinks;
+                asSetting.recursive = adir ? adir->getRecursive() : false;
+                asSetting.hidden = adir ? adir->getHidden() : defHidden;
+                asSetting.rescanResource = true;
+                asSetting.async = true;
+                asSetting.mergeOptions(config, path);
 
                 // file is directory
-                if (mask & IN_ISDIR) {
+                if (isDir) {
                     if (mask & (IN_CREATE | IN_MOVED_TO)) {
                         recheckNonexistingMonitors(wd, wdObj);
                     }
 
-                    if (adir && adir->getRecursive() && (mask & IN_CREATE)) {
-                        if (adir->getHidden() || name.at(0) != '.') {
+                    if (asSetting.recursive && (mask & IN_CREATE)) {
+                        if (!content->isHiddenFile(dirEnt, isDir, asSetting)) {
                             log_debug("Detected new dir, adding to inotify: {}", path.c_str());
-                            auto dirEnt = fs::directory_entry(path, ec);
-                            if (!ec) {
-                                monitorUnmonitorRecursive(dirEnt, false, adir, false, config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS));
-                            } else {
-                                log_error("Failed to read {}: {}", path.c_str(), ec.message());
-                            }
+                            monitorUnmonitorRecursive(dirEnt, false, adir, false, asSetting.followSymlinks);
                         } else {
-                            log_debug("Detected new dir, irgnoring because it's hidden: {}", path.c_str());
+                            log_debug("Detected new dir, ignoring because it's hidden: {}", path.c_str());
                         }
                     }
                 }
@@ -228,8 +252,10 @@ void AutoscanInotify::threadProc()
 
                         // deleted
                         if (mask & (IN_DELETE_SELF | IN_MOVE_SELF | IN_UNMOUNT)) {
-                            if (mask & IN_MOVE_SELF)
+                            if (!(mask & IN_MOVE_SELF)) {
+                                log_debug("removing watch {}", path.c_str());
                                 inotify->removeWatch(wd);
+                            }
                             auto watch = getStartPoint(wdObj);
                             if (watch && adir->persistent()) {
                                 monitorNonexisting(path, watch->getAutoscanDirectory());
@@ -237,29 +263,22 @@ void AutoscanInotify::threadProc()
                             }
                         }
 
-                        int objectID = database->findObjectIDByPath(path, !(mask & IN_ISDIR));
-                        if (objectID != INVALID_OBJECT_ID)
-                            content->removeObject(adir, objectID, path, !(mask & IN_MOVED_TO));
+                        auto object = database->findObjectByPath(path, !(mask & IN_ISDIR) ? DbFileType::File : DbFileType::Directory);
+                        if (object)
+                            content->removeObject(adir, object, path, !(mask & IN_MOVED_TO));
                     }
                     // new file
                     if (mask & (IN_CLOSE_WRITE | IN_MOVED_TO | IN_CREATE)) {
                         log_debug("Adding {}", path.c_str());
-                        auto dirEnt = fs::directory_entry(path, ec);
-                        if (!ec) {
-                            AutoScanSetting asSetting;
-                            asSetting.adir = adir;
-                            asSetting.followSymlinks = config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS);
-                            asSetting.recursive = adir->getRecursive();
-                            asSetting.hidden = adir->getHidden();
-                            asSetting.rescanResource = true;
-                            asSetting.mergeOptions(config, path);
-                            // path, recursive, async, hidden, rescanResource, low priority, cancellable
-                            content->addFile(dirEnt, adir->getLocation(), asSetting, true, true, false);
-                            if (mask & IN_ISDIR) {
-                                monitorUnmonitorRecursive(dirEnt, false, adir, false, asSetting.followSymlinks);
+                        // dirEnt, path, rootPath, settings, lowPriority, cancellable
+                        content->addFile(dirEnt, adir->getLocation(), asSetting, true, false);
+                        if (isDir) {
+                            int wdPath = monitorUnmonitorRecursive(dirEnt, false, adir, false, asSetting.followSymlinks);
+                            if ((mask & IN_MOVED_TO) && wdPath > INOTIFY_ROOT) {
+                                auto wdObjPath = watches.at(wdPath);
+                                log_debug("Resetting {} to {}", wdObjPath->getPath().c_str(), path.c_str());
+                                wdObjPath->setPath(path);
                             }
-                        } else {
-                            log_error("Failed to read {}: {}", path.c_str(), ec.message());
                         }
                     }
                 }
@@ -268,6 +287,7 @@ void AutoscanInotify::threadProc()
                     removeDescendants(wd);
                     watches.erase(wd);
                 }
+                // lock.unlock();
             }
         } catch (const std::runtime_error& e) {
             log_error("Inotify thread caught exception: {}", e.what());
@@ -312,7 +332,7 @@ int AutoscanInotify::watchPathForMoves(const fs::path& path, int wd)
 int AutoscanInotify::addMoveWatch(const fs::path& path, int removeWd, int parentWd)
 {
     int wd = inotify->addWatch(path, events);
-    if (wd >= 0) {
+    if (wd > INOTIFY_ROOT) {
         std::shared_ptr<Wd> wdObj;
         try {
             // find
@@ -320,7 +340,7 @@ int AutoscanInotify::addMoveWatch(const fs::path& path, int removeWd, int parent
 
             // alreadyThere
             int parentWdSet = wdObj->getParentWd();
-            if (parentWdSet >= 0) {
+            if (parentWdSet > INOTIFY_ROOT) {
                 if (parentWd != parentWdSet) {
                     log_debug("error: parentWd doesn't match wd: {}, parent is: {}, should be: {}", wd, parentWdSet, parentWd);
                     wdObj->setParentWd(parentWd);
@@ -342,36 +362,24 @@ int AutoscanInotify::addMoveWatch(const fs::path& path, int removeWd, int parent
 
 void AutoscanInotify::monitorNonexisting(const fs::path& path, const std::shared_ptr<AutoscanDirectory>& adir)
 {
-    auto pathAr = splitString(path.string(), DIR_SEPARATOR);
-    recheckNonexistingMonitor(-1, pathAr, adir);
+    recheckNonexistingMonitor(INOTIFY_ROOT, path, adir);
 }
 
-void AutoscanInotify::recheckNonexistingMonitor(int curWd, const std::vector<std::string>& pathAr, const std::shared_ptr<AutoscanDirectory>& adir)
+void AutoscanInotify::recheckNonexistingMonitor(int curWd, const fs::path& nonExistingPath, const std::shared_ptr<AutoscanDirectory>& adir)
 {
+    std::error_code ec;
     bool first = true;
-    for (std::size_t i = pathAr.size() + 1; i > 0;) {
-        i--;
-        std::ostringstream buf;
-        if (i == 0)
-            buf << DIR_SEPARATOR;
-        else {
-            for (std::size_t j = 0; j < i; j++) {
-                buf << DIR_SEPARATOR << pathAr.at(j);
-                //                log_debug("adding: {}", pathAr->get(j)->data);
-            }
-        }
-
-        fs::path path = buf.str();
-        bool pathExists = fs::is_directory(path);
-        //        log_debug("checking {}: {}", path.c_str(), pathExists);
+    for (fs::path path = nonExistingPath; path != "/"; path = path.parent_path()) {
+        bool pathExists = fs::is_directory(path, ec);
+        log_debug("recheckNonexistingMonitor: Checking {}: {}", path.c_str(), pathExists);
         if (pathExists) {
-            if (curWd != -1)
-                removeNonexistingMonitor(curWd, watches.at(curWd), pathAr);
+            if (curWd > INOTIFY_ROOT)
+                removeNonexistingMonitor(curWd, watches.at(curWd), nonExistingPath);
             if (first) {
                 monitorDirectory(path, adir, true);
                 content->handlePersistentAutoscanRecreate(adir);
             } else {
-                monitorDirectory(path, adir, false, &pathAr);
+                monitorDirectory(path, adir, false, true, nonExistingPath);
             }
             break;
         }
@@ -406,14 +414,14 @@ void AutoscanInotify::checkMoveWatches(int wd, const std::shared_ptr<Wd>& wdObj)
                 auto watchToRemove = getStartPoint(wdToRemove);
                 if (watchToRemove) {
                     auto adir = watchToRemove->getAutoscanDirectory();
-                    if (adir->persistent()) {
+                    if (adir && adir->persistent()) {
                         monitorNonexisting(path, adir);
                         content->handlePeristentAutoscanRemove(adir);
                     }
 
-                    int objectID = database->findObjectIDByPath(path, true);
-                    if (objectID != INVALID_OBJECT_ID)
-                        content->removeObject(adir, objectID, path, false);
+                    auto object = database->findObjectByPath(path, DbFileType::File);
+                    if (object)
+                        content->removeObject(adir, object, path, false);
                 }
             } catch (const std::out_of_range&) {
             } // Not found in map
@@ -428,22 +436,22 @@ void AutoscanInotify::recheckNonexistingMonitors(int wd, const std::shared_ptr<W
     for (auto&& watch : *wdWatches) {
         if (watch->getType() == WatchType::Autoscan) {
             auto watchAs = std::static_pointer_cast<WatchAutoscan>(watch);
-            auto pathAr = watchAs->getNonexistingPathArray();
-            if (!pathAr.empty()) {
-                recheckNonexistingMonitor(wd, pathAr, watchAs->getAutoscanDirectory());
+            auto nonExistingPath = watchAs->getNonExistingPath();
+            if (!nonExistingPath.empty()) {
+                recheckNonexistingMonitor(wd, nonExistingPath, watchAs->getAutoscanDirectory());
             }
         }
     }
 }
 
-void AutoscanInotify::removeNonexistingMonitor(int wd, const std::shared_ptr<Wd>& wdObj, const std::vector<std::string>& pathAr)
+void AutoscanInotify::removeNonexistingMonitor(int wd, const std::shared_ptr<Wd>& wdObj, const fs::path& nonExistingPath)
 {
     auto&& wdWatches = wdObj->getWdWatches();
     auto it = std::find_if(wdWatches->begin(), wdWatches->end(),
         [&](auto&& watch) {
             if (watch->getType() == WatchType::Autoscan) {
                 auto watchAs = std::static_pointer_cast<WatchAutoscan>(watch);
-                return watchAs->getNonexistingPathArray() == pathAr;
+                return watchAs->getNonExistingPath() == nonExistingPath;
             }
             return false; });
     if (it != wdWatches->end()) {
@@ -456,66 +464,77 @@ void AutoscanInotify::removeNonexistingMonitor(int wd, const std::shared_ptr<Wd>
     }
 }
 
-void AutoscanInotify::monitorUnmonitorRecursive(const fs::directory_entry& startPath, bool unmonitor, const std::shared_ptr<AutoscanDirectory>& adir, bool startPoint, bool followSymlinks)
+int AutoscanInotify::monitorUnmonitorRecursive(const fs::directory_entry& startPath, bool unmonitor, const std::shared_ptr<AutoscanDirectory>& adir, bool isStartPoint, bool followSymlinks)
 {
+    int result = INOTIFY_ROOT;
     if (unmonitor)
         unmonitorDirectory(startPath.path(), adir);
     else {
-        bool ok = (monitorDirectory(startPath.path(), adir, startPoint) > 0);
-        if (!ok)
-            return;
+        result = monitorDirectory(startPath.path(), adir, isStartPoint);
+        if (result <= INOTIFY_ROOT)
+            return INOTIFY_ROOT;
     }
 
     std::error_code ec;
     if (!startPath.exists(ec) || !startPath.is_directory(ec)) {
         log_warning("Could not open {}: {}", startPath.path().c_str(), ec.message());
-        return;
+        return INOTIFY_ROOT;
     }
     auto dIter = fs::directory_iterator(startPath, ec);
     if (ec) {
         log_error("monitorUnmonitorRecursive: Failed to iterate {}, {}", startPath.path().c_str(), ec.message());
-        return;
+        return INOTIFY_ROOT;
     }
     for (auto&& dirEnt : dIter) {
         if (shutdownFlag)
             break;
 
-        if (!followSymlinks && dirEnt.is_symlink(ec)) {
-            log_debug("link {} skipped", dirEnt.path().c_str());
+        AutoScanSetting asSetting;
+        asSetting.adir = adir;
+        asSetting.followSymlinks = adir ? adir->getFollowSymlinks() : defFollowSymlinks;
+        asSetting.recursive = adir ? adir->getRecursive() : false;
+        asSetting.hidden = adir ? adir->getHidden() : defHidden;
+        asSetting.rescanResource = true;
+        asSetting.async = true;
+        asSetting.mergeOptions(config, dirEnt.path());
+
+        if (content->isHiddenFile(dirEnt, dirEnt.is_directory(ec), asSetting)) {
+            log_debug("Hidden file {} skipped", dirEnt.path().c_str());
             continue;
         }
 
-        if (dirEnt.is_directory(ec) && adir->getRecursive()) {
+        if (dirEnt.is_directory(ec) && asSetting.recursive) {
             monitorUnmonitorRecursive(dirEnt, unmonitor, adir, false, followSymlinks);
         }
 
         if (ec) {
-            log_error("monitorUnmonitorRecursive: Failed to read {}, {}", dirEnt.path().c_str(), ec.message());
+            log_error("AutoscanInotify::monitorUnmonitorRecursive {}: Failed to read {}, {}", startPath.path().c_str(), dirEnt.path().c_str(), ec.message());
         }
     }
+    return result;
 }
 
-int AutoscanInotify::monitorDirectory(const fs::path& path, const std::shared_ptr<AutoscanDirectory>& adir, bool startPoint, const std::vector<std::string>* pathArray)
+int AutoscanInotify::monitorDirectory(const fs::path& path, const std::shared_ptr<AutoscanDirectory>& adir, bool isStartPoint, bool hasNonExisting, const fs::path& nonExistingPath)
 {
     int wd = inotify->addWatch(path, events);
-    if (wd < 0) {
-        if (startPoint && adir->persistent()) {
+    if (wd <= INOTIFY_ROOT) {
+        if (isStartPoint && adir && adir->persistent()) {
             monitorNonexisting(path, adir);
         }
     } else {
         bool alreadyWatching = false;
         int parentWd = INOTIFY_UNKNOWN_PARENT_WD;
-        if (startPoint)
+        if (isStartPoint)
             parentWd = watchPathForMoves(path, wd);
 
         std::shared_ptr<Wd> wdObj;
         try {
             wdObj = watches.at(wd);
-            if (parentWd >= 0 && wdObj->getParentWd() < 0) {
+            if (parentWd > INOTIFY_ROOT && wdObj->getParentWd() <= INOTIFY_ROOT) {
                 wdObj->setParentWd(parentWd);
             }
 
-            if (!pathArray)
+            if (!hasNonExisting)
                 alreadyWatching = getAppropriateAutoscan(wdObj, adir) != nullptr;
 
             // should we check for already existing "nonexisting" watches?
@@ -526,16 +545,16 @@ int AutoscanInotify::monitorDirectory(const fs::path& path, const std::shared_pt
         }
 
         if (!alreadyWatching) {
-            auto watch = std::make_shared<WatchAutoscan>(startPoint, adir);
-            if (pathArray) {
-                watch->setNonexistingPathArray(*pathArray);
+            auto watch = std::make_shared<WatchAutoscan>(isStartPoint, adir);
+            if (hasNonExisting) {
+                watch->setNonExistingPath(nonExistingPath);
             }
-            wdObj->addWatch(move(watch));
+            wdObj->addWatch(std::move(watch));
 
-            if (!startPoint) {
+            if (!isStartPoint && adir) {
                 int startPointWd = inotify->addWatch(adir->getLocation(), events);
                 log_debug("getting start point for {} -> {} wd={}", path.c_str(), adir->getLocation().c_str(), startPointWd);
-                if (wd >= 0)
+                if (wd > INOTIFY_ROOT)
                     addDescendant(startPointWd, wd, adir);
             }
         }
@@ -550,7 +569,7 @@ void AutoscanInotify::unmonitorDirectory(const fs::path& path, const std::shared
     // this should not add a new watch, because it should be already watched
     int wd = inotify->addWatch(path, events);
 
-    if (wd < 0) {
+    if (wd <= INOTIFY_ROOT) {
         // doesn't seem to be monitored currently
         log_debug("unmonitorDirectory called, but it isn't monitored? ({})", path.c_str());
         return;
@@ -583,7 +602,7 @@ std::shared_ptr<AutoscanInotify::WatchAutoscan> AutoscanInotify::getAppropriateA
     for (auto&& watch : *wdWatches) {
         if (watch->getType() == WatchType::Autoscan) {
             auto watchAs = std::static_pointer_cast<WatchAutoscan>(watch);
-            if (watchAs->getNonexistingPathArray().empty() && watchAs->getAutoscanDirectory()->getLocation() == adir->getLocation()) {
+            if (watchAs->getNonExistingPath().empty() && watchAs->getAutoscanDirectory()->getLocation() == adir->getLocation()) {
                 return watchAs;
             }
         }
@@ -599,7 +618,7 @@ std::shared_ptr<AutoscanInotify::WatchAutoscan> AutoscanInotify::getAppropriateA
     for (auto&& watch : *wdWatches) {
         if (watch->getType() == WatchType::Autoscan) {
             auto watchAs = std::static_pointer_cast<WatchAutoscan>(watch);
-            if (watchAs->getNonexistingPathArray().empty()) {
+            if (watchAs->getNonExistingPath().empty()) {
                 fs::path testLocation = watchAs->getAutoscanDirectory()->getLocation();
                 if (isSubDir(path, testLocation)) {
                     if (!pathBestMatch.empty()) {
@@ -653,7 +672,7 @@ void AutoscanInotify::removeWatchMoves(int wd)
             }
         }
         checkWd = wdObj->getParentWd();
-    } while (checkWd >= 0);
+    } while (checkWd > INOTIFY_ROOT);
 }
 
 bool AutoscanInotify::removeFromWdObj(const std::shared_ptr<Wd>& wdObj, const std::shared_ptr<Watch>& toRemove)
@@ -678,7 +697,7 @@ std::shared_ptr<AutoscanInotify::WatchAutoscan> AutoscanInotify::getStartPoint(c
     for (auto&& watch : *wdWatches) {
         if (watch->getType() == WatchType::Autoscan) {
             auto watchAs = std::static_pointer_cast<WatchAutoscan>(watch);
-            if (watchAs->isStartPoint())
+            if (watchAs->getIsStartPoint())
                 return watchAs;
         }
     }
@@ -687,7 +706,7 @@ std::shared_ptr<AutoscanInotify::WatchAutoscan> AutoscanInotify::getStartPoint(c
 
 void AutoscanInotify::addDescendant(int startPointWd, int addWd, const std::shared_ptr<AutoscanDirectory>& adir)
 {
-    //    log_debug("called for {}, (adir->path={}); adding {}", startPointWd, adir->getLocation().c_str(), addWd);
+    // log_debug("called for {}, (adir->path={}); adding {}", startPointWd, adir->getLocation().c_str(), addWd);
     std::shared_ptr<Wd> wdObj;
     try {
         wdObj = watches.at(startPointWd);
@@ -697,13 +716,13 @@ void AutoscanInotify::addDescendant(int startPointWd, int addWd, const std::shar
     if (!wdObj)
         return;
 
-    //   log_debug("found wdObj");
+    // log_debug("found wdObj");
     auto watch = getAppropriateAutoscan(wdObj, adir);
     if (!watch)
         return;
     // log_debug("adding descendant");
     watch->addDescendant(addWd);
-    // log_debug("added descendant to {} (adir->path={}): {}; now: {}", startPointWd, adir->getLocation(), addWd, fmt::join(*(watch->getDescendants()), ","));
+    // log_debug("added descendant to {} (adir->path={}): {}; now: {}", startPointWd, adir->getLocation().c_str(), addWd, fmt::join(watch->getDescendants(), ","));
 }
 
 void AutoscanInotify::removeDescendants(int wd)
