@@ -38,8 +38,10 @@
 
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
+#include "config/config.h"
 #include "content/autoscan.h"
 #include "content/content_manager.h"
+#include "content/layout/box_layout.h"
 #include "metadata/metadata_handler.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
@@ -59,64 +61,70 @@ BuiltinLayout::BuiltinLayout(std::shared_ptr<ContentManager> content)
     , config(this->content->getContext()->getConfig())
     , genreMap(this->config->getDictionaryOption(CFG_IMPORT_SCRIPTING_IMPORT_GENRE_MAP))
 {
-    container["Video"] = std::make_shared<CdsContainer>("Video");
+    auto blOption = config->getBoxLayoutListOption(CFG_BOXLAYOUT_BOX);
+    for (auto&& bl : blOption->getArrayCopy()) {
+        container[bl->getKey()] = std::make_shared<CdsContainer>(bl->getTitle());
+    }
+
+    container["Video"] = container.at("Video/videoRoot");
     container["Video"]->addMetaData(M_CONTENT_CLASS, UPNP_CLASS_VIDEO_ITEM);
-    container["All Video"] = std::make_shared<CdsContainer>("All Video");
-    chain["/Video/All Video"] = this->content->addContainerTree({ container["Video"], container["All Video"] });
-    container["Video/Year"] = std::make_shared<CdsContainer>("Year");
-    chain["/Video/Year"] = this->content->addContainerTree({ container["Video"], container["Video/Year"] });
-    container["Video/Date"] = std::make_shared<CdsContainer>("Date");
-    chain["/Video/Date"] = this->content->addContainerTree({ container["Video"], container["Video/Date"] });
-    container["Video/Directories"] = std::make_shared<CdsContainer>("Directories");
-    chain["/Video/Directories"] = this->content->addContainerTree({ container["Video"], container["Video/Directories"] });
+    container["All Video"] = container.at("Video/allVideo");
+    chain["/Video/All Video"] = this->content->addContainerTree({ container["Video"], container["All Video"] }, nullptr);
+    container["Video/Year"] = container.at("Video/allYears");
+    chain["/Video/Year"] = this->content->addContainerTree({ container["Video"], container["Video/Year"] }, nullptr);
+    container["Video/Date"] = container.at("Video/allDates");
+    chain["/Video/Date"] = this->content->addContainerTree({ container["Video"], container["Video/Date"] }, nullptr);
+    container["Video/Directories"] = container.at("Video/allDirectories");
+    chain["/Video/Directories"] = this->content->addContainerTree({ container["Video"], container["Video/Directories"] }, nullptr);
 
-    container["Photos"] = std::make_shared<CdsContainer>("Photos");
+    container["Photos"] = container.at("Image/imageRoot");
     container["Photos"]->addMetaData(M_CONTENT_CLASS, UPNP_CLASS_IMAGE_ITEM);
-    container["All Photos"] = std::make_shared<CdsContainer>("All Photos");
-    chain["/Photos/All Photos"] = this->content->addContainerTree({ container["Photos"], container["All Photos"] });
-    container["Photos/Year"] = std::make_shared<CdsContainer>("Year");
-    chain["/Photos/Year"] = this->content->addContainerTree({ container["Photos"], container["Photos/Year"] });
-    container["Photos/Date"] = std::make_shared<CdsContainer>("Date");
-    chain["/Photos/Date"] = this->content->addContainerTree({ container["Photos"], container["Photos/Date"] });
-    container["Photos/Directories"] = std::make_shared<CdsContainer>("Directories");
-    chain["/Photos/Directories"] = this->content->addContainerTree({ container["Photos"], container["Photos/Directories"] });
+    container["All Photos"] = container.at("Image/allImages");
+    chain["/Photos/All Photos"] = this->content->addContainerTree({ container["Photos"], container["All Photos"] }, nullptr);
+    container["Photos/Year"] = container.at("Image/allYears");
+    chain["/Photos/Year"] = this->content->addContainerTree({ container["Photos"], container["Photos/Year"] }, nullptr);
+    container["Photos/Date"] = container.at("Image/allDates");
+    chain["/Photos/Date"] = this->content->addContainerTree({ container["Photos"], container["Photos/Date"] }, nullptr);
+    container["Photos/Directories"] = container.at("Image/allDirectories");
+    chain["/Photos/Directories"] = this->content->addContainerTree({ container["Photos"], container["Photos/Directories"] }, nullptr);
 
-    container["Audio"] = std::make_shared<CdsContainer>("Audio");
+    container["Audio"] = container.at("Audio/audioRoot");
     container["Audio"]->addMetaData(M_CONTENT_CLASS, UPNP_CLASS_AUDIO_ITEM);
-    container["All Audio"] = std::make_shared<CdsContainer>("All Audio");
-    container["All Songs"] = std::make_shared<CdsContainer>("All Songs");
-    chain["/Audio/All Audio"] = this->content->addContainerTree({ container["Audio"], container["All Audio"] });
-    container["All - full name"] = std::make_shared<CdsContainer>("All - full name");
-    chain["/Audio/All - full name"] = this->content->addContainerTree({ container["Audio"], container["All - full name"] });
-    container["Albums"] = std::make_shared<CdsContainer>("Albums");
-    chain["/Audio/Albums"] = this->content->addContainerTree({ container["Audio"], container["Albums"] });
-    container["Artists"] = std::make_shared<CdsContainer>("Artists");
-    chain["/Audio/Artists"] = this->content->addContainerTree({ container["Audio"], container["Artists"] });
-    container["Genres"] = std::make_shared<CdsContainer>("Genres");
-    chain["/Audio/Genres"] = this->content->addContainerTree({ container["Audio"], container["Genres"] });
-    container["Composers"] = std::make_shared<CdsContainer>("Composers");
-    chain["/Audio/Composers"] = this->content->addContainerTree({ container["Audio"], container["Composers"] });
-    container["Year"] = std::make_shared<CdsContainer>("Year");
-    chain["/Audio/Year"] = this->content->addContainerTree({ container["Audio"], container["Year"] });
-    container["Audio/Directories"] = std::make_shared<CdsContainer>("Directories");
-    chain["/Audio/Directories"] = this->content->addContainerTree({ container["Audio"], container["Audio/Directories"] });
+    container["All Audio"] = container.at("Audio/allAudio");
+    container["All Songs"] = container.at("Audio/allSongs");
+    chain["/Audio/All Audio"] = this->content->addContainerTree({ container["Audio"], container["All Audio"] }, nullptr);
+    container["All - full name"] = container.at("Audio/allTracks");
+    chain["/Audio/All - full name"] = this->content->addContainerTree({ container["Audio"], container["All - full name"] }, nullptr);
+    container["Albums"] = container.at("Audio/allAlbums");
+    chain["/Audio/Albums"] = this->content->addContainerTree({ container["Audio"], container["Albums"] }, nullptr);
+    container["Artists"] = container.at("Audio/allArtists");
+    chain["/Audio/Artists"] = this->content->addContainerTree({ container["Audio"], container["Artists"] }, nullptr);
+    container["Genres"] = container.at("Audio/allGenres");
+    chain["/Audio/Genres"] = this->content->addContainerTree({ container["Audio"], container["Genres"] }, nullptr);
+    container["Composers"] = container.at("Audio/allComposers");
+    chain["/Audio/Composers"] = this->content->addContainerTree({ container["Audio"], container["Composers"] }, nullptr);
+    container["Year"] = container.at("Audio/allYears");
+    chain["/Audio/Year"] = this->content->addContainerTree({ container["Audio"], container["Year"] }, nullptr);
+    container["Audio/Directories"] = container.at("Audio/allDirectories");
+    chain["/Audio/Directories"] = this->content->addContainerTree({ container["Audio"], container["Audio/Directories"] }, nullptr);
+    container["Artist Chronology"] = container.at("Audio/artistChronology");
 
 #ifdef ONLINE_SERVICES
     if (config->getBoolOption(CFG_ONLINE_CONTENT_ATRAILERS_ENABLED)) {
-        container["Online Services"] = std::make_shared<CdsContainer>("Online Services");
+        container["Online Services"] = container.at("Trailer/trailerRoot");
     }
 
 #ifdef ATRAILERS
     if (config->getBoolOption(CFG_ONLINE_CONTENT_ATRAILERS_ENABLED)) {
-        container["Apple"] = std::make_shared<CdsContainer>("Apple Trailers");
-        container["Apple/All Trailers"] = std::make_shared<CdsContainer>("All Trailers");
-        container["Apple/Genres"] = std::make_shared<CdsContainer>("Genres");
-        container["Apple/Release Date"] = std::make_shared<CdsContainer>("Release Date");
-        container["Apple/Post Date"] = std::make_shared<CdsContainer>("Post Date");
-        chain["/Online Services/Apple/All Trailers"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/All Trailers"] });
-        chain["/Online Services/Apple/Genres"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/Genres"] });
-        chain["/Online Services/Apple/Release Date"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/Release Date"] });
-        chain["/Online Services/Apple/Post Date"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/Post Date"] });
+        container["Apple"] = container.at("Trailer/appleTrailers");
+        container["Apple/All Trailers"] = container.at("Trailer/allTrailers");
+        container["Apple/Genres"] = container.at("Trailer/allGenres");
+        container["Apple/Release Date"] = container.at("Trailer/relDate");
+        container["Apple/Post Date"] = container.at("Trailer/postDate");
+        chain["/Online Services/Apple/All Trailers"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/All Trailers"] }, nullptr);
+        chain["/Online Services/Apple/Genres"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/Genres"] }, nullptr);
+        chain["/Online Services/Apple/Release Date"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/Release Date"] }, nullptr);
+        chain["/Online Services/Apple/Post Date"] = this->content->addContainerTree({ container["Online Services"], container["Apple"], container["Apple/Post Date"] }, nullptr);
     }
 #endif
 #endif
@@ -155,13 +163,13 @@ void BuiltinLayout::getDir(const std::shared_ptr<CdsObject>& obj, const fs::path
             if (segment != "/" && segment != ".." && !segment.empty())
                 dirVect.push_back(std::make_shared<CdsContainer>(segment.string()));
         }
-        auto id = content->addContainerTree(dirVect);
+        auto id = content->addContainerTree(dirVect, obj);
         dirVect[dirVect.size() - 1]->setClass(upnpClass);
         add(obj, id);
     }
 }
 
-void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     auto f2i = StringConverter::f2i(config);
     auto id = chain["/Video/All Video"];
@@ -197,7 +205,7 @@ void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const fs::pa
             ct.push_back(container["Video/Year"]);
             ct.push_back(std::make_shared<CdsContainer>(year));
             ct.push_back(std::make_shared<CdsContainer>(month));
-            id = content->addContainerTree(ct);
+            id = content->addContainerTree(ct, obj);
             add(obj, id);
         }
 
@@ -209,14 +217,14 @@ void BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const fs::pa
         ct.push_back(container["Video"]);
         ct.push_back(container["Video/Date"]);
         ct.push_back(std::make_shared<CdsContainer>(date));
-        id = content->addContainerTree(ct);
+        id = content->addContainerTree(ct, obj);
         add(obj, id);
     }
 
     getDir(obj, rootpath, "Video", "Video/Directories", getValueOrDefault(containerMap, AutoscanMediaMode::Video, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Video)));
 }
 
-void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     auto f2i = StringConverter::f2i(config);
     log_debug("add image file {}", obj->getLocation().string());
@@ -251,7 +259,7 @@ void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const fs::pa
             ct.push_back(container["Photos/Year"]);
             ct.push_back(std::make_shared<CdsContainer>(year));
             ct.push_back(std::make_shared<CdsContainer>(month));
-            id = content->addContainerTree(ct);
+            id = content->addContainerTree(ct, obj);
             add(obj, id);
         }
 
@@ -263,14 +271,14 @@ void BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const fs::pa
         ct.push_back(container["Photos"]);
         ct.push_back(container["Photos/Date"]);
         ct.push_back(std::make_shared<CdsContainer>(date));
-        id = content->addContainerTree(ct);
+        id = content->addContainerTree(ct, obj);
         add(obj, id);
     }
 
     getDir(obj, rootpath, "Photos", "Photos/Directories", getValueOrDefault(containerMap, AutoscanMediaMode::Image, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Image)));
 }
 
-void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     auto f2i = StringConverter::f2i(config);
 
@@ -359,7 +367,7 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::pa
     arc.push_back(container["Artists"]);
     arc.push_back(artistContainer);
     arc.push_back(container["All Songs"]);
-    id = content->addContainerTree(arc);
+    id = content->addContainerTree(arc, obj);
     add(obj, id);
 
     std::string temp;
@@ -382,7 +390,7 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::pa
     alc.push_back(container["Artists"]);
     alc.push_back(artistContainer);
     alc.push_back(albumContainer);
-    id = content->addContainerTree(alc);
+    id = content->addContainerTree(alc, obj);
     add(obj, id);
 
     albumContainer->setSearchable(true);
@@ -390,14 +398,14 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::pa
     allc.push_back(container["Audio"]);
     allc.push_back(container["Albums"]);
     allc.push_back(std::move(albumContainer));
-    id = content->addContainerTree(allc);
+    id = content->addContainerTree(allc, obj);
     add(obj, id);
 
     std::vector<std::shared_ptr<CdsObject>> ct;
     ct.push_back(container["Audio"]);
     ct.push_back(container["Genres"]);
     ct.push_back(std::make_shared<CdsContainer>(genre, UPNP_CLASS_MUSIC_GENRE));
-    id = content->addContainerTree(ct);
+    id = content->addContainerTree(ct, obj);
     add(obj, id);
 
     auto composerContainer = std::make_shared<CdsContainer>(composer, UPNP_CLASS_MUSIC_COMPOSER);
@@ -406,7 +414,7 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::pa
     cc.push_back(container["Audio"]);
     cc.push_back(container["Composers"]);
     cc.push_back(std::move(composerContainer));
-    id = content->addContainerTree(cc);
+    id = content->addContainerTree(cc, obj);
     add(obj, id);
 
     auto yearContainer = std::make_shared<CdsContainer>(date);
@@ -415,6 +423,7 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::pa
     yt.push_back(container["Audio"]);
     yt.push_back(container["Year"]);
     yt.push_back(std::move(yearContainer));
+    id = content->addContainerTree(yt, obj);
     add(obj, id);
 
     obj->setTitle(fmt::format("{}{}", temp, title));
@@ -423,15 +432,42 @@ void BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const fs::pa
     std::vector<std::shared_ptr<CdsObject>> all;
     all.push_back(container["Audio"]);
     all.push_back(container["Artists"]);
-    all.push_back(std::move(artistContainer));
+    all.push_back(artistContainer);
     all.push_back(container["All - full name"]);
-    id = content->addContainerTree(all);
+    id = content->addContainerTree(all, obj);
     add(obj, id);
 
     obj->setTitle(title);
     getDir(obj, rootpath, "Audio", "Audio/Directories", getValueOrDefault(containerMap, AutoscanMediaMode::Audio, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Audio)));
+
+    artistContainer->setSearchable(false);
+    std::vector<std::shared_ptr<CdsObject>> chronology;
+    chronology.push_back(container["Audio"]);
+    chronology.push_back(container["Artists"]);
+    chronology.push_back(std::move(artistContainer));
+    chronology.push_back(container["Artist Chronology"]);
+    chronology.push_back(std::make_shared<CdsContainer>(date + " - " + album, UPNP_CLASS_MUSIC_ALBUM));
+    id = content->addContainerTree(chronology, obj);
+    add(obj, id);
 }
 
+#ifdef ONLINE_SERVICES
+void BuiltinLayout::addTrailer(const std::shared_ptr<CdsObject>& obj, OnlineServiceType serviceType, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+{
+    switch (serviceType) {
+#ifdef ATRAILERS
+    case OnlineServiceType::OS_ATrailers:
+        addATrailers(obj);
+        break;
+#endif
+    case OnlineServiceType::OS_Max:
+    default:
+        log_warning("No handler for service type");
+        break;
+    }
+}
+#endif
+
 #ifdef ATRAILERS
 void BuiltinLayout::addATrailers(const std::shared_ptr<CdsObject>& obj)
 {
@@ -462,7 +498,7 @@ void BuiltinLayout::addATrailers(const std::shared_ptr<CdsObject>& obj)
         ct.push_back(container["Apple"]);
         ct.push_back(container["Genres"]);
         ct.push_back(std::make_shared<CdsContainer>(genre));
-        auto id = content->addContainerTree(ct);
+        auto id = content->addContainerTree(ct, obj);
         add(obj, id);
     }
 
@@ -473,7 +509,7 @@ void BuiltinLayout::addATrailers(const std::shared_ptr<CdsObject>& obj)
         ct.push_back(container["Apple"]);
         ct.push_back(container["Release Date"]);
         ct.push_back(std::make_shared<CdsContainer>(temp.substr(0, 7)));
-        auto id = content->addContainerTree(ct);
+        auto id = content->addContainerTree(ct, obj);
         add(obj, id);
     }
 
@@ -484,7 +520,7 @@ void BuiltinLayout::addATrailers(const std::shared_ptr<CdsObject>& obj)
         ct.push_back(container["Apple"]);
         ct.push_back(container["Post Date"]);
         ct.push_back(std::make_shared<CdsContainer>(temp.substr(0, 7)));
-        auto id = content->addContainerTree(ct);
+        auto id = content->addContainerTree(ct, obj);
         add(obj, id);
     }
 }
@@ -499,47 +535,3 @@ std::string BuiltinLayout::mapGenre(const std::string& genre)
     }
     return genre;
 }
-
-void BuiltinLayout::processCdsObject(const std::shared_ptr<CdsObject>& obj, const fs::path& rootpath, const std::string& contentType, const std::map<AutoscanMediaMode, std::string>& containerMap)
-{
-    log_debug("Process CDS Object: {}", obj->getTitle());
-    auto clone = CdsObject::createObject(obj->getObjectType());
-    obj->copyTo(clone);
-    clone->setVirtual(true);
-
-#ifdef ONLINE_SERVICES
-    if (clone->getFlag(OBJECT_FLAG_ONLINE_SERVICE)) {
-        auto service = OnlineServiceType(std::stoi(clone->getAuxData(ONLINE_SERVICE_AUX_ID)));
-
-        switch (service) {
-#ifdef ATRAILERS
-        case OnlineServiceType::OS_ATrailers:
-            addATrailers(clone);
-            break;
-#endif
-        case OnlineServiceType::OS_Max:
-        default:
-            log_warning("No handler for service type");
-            break;
-        }
-    } else {
-#endif
-
-        auto objCls = std::static_pointer_cast<CdsItem>(obj)->getClass();
-        if (contentType == CONTENT_TYPE_OGG) {
-            if (obj->getFlag(OBJECT_FLAG_OGG_THEORA))
-                addVideo(clone, rootpath, containerMap);
-            else
-                addAudio(clone, rootpath, containerMap);
-        } else if (startswith(objCls, UPNP_CLASS_VIDEO_ITEM)) {
-            addVideo(clone, rootpath, containerMap);
-        } else if (startswith(objCls, UPNP_CLASS_IMAGE_ITEM)) {
-            addImage(clone, rootpath, containerMap);
-        } else if (startswith(objCls, UPNP_CLASS_AUDIO_ITEM) && contentType != CONTENT_TYPE_PLAYLIST) {
-            addAudio(clone, rootpath, containerMap);
-        }
-
-#ifdef ONLINE_SERVICES
-    }
-#endif
-}
