@@ -42,8 +42,11 @@
 #include "cds/cds_item.h"
 #include "config/config_definition.h"
 #include "config/config_setup.h"
+#include "config/setup/config_setup_autoscan.h"
+#include "config/setup/config_setup_boxlayout.h"
 #include "content/autoscan.h"
 #include "content/content_manager.h"
+#include "content/layout/box_layout.h"
 #include "database/database.h"
 #include "js_functions.h"
 #include "metadata/metadata_handler.h"
@@ -170,21 +173,21 @@ void Script::setIntProperty(const std::string& name, int value)
 
 /* **************** */
 
-Script::Script(const std::shared_ptr<ContentManager>& content,
-    const std::shared_ptr<ScriptingRuntime>& runtime, const std::string& name,
-    std::string objName, std::unique_ptr<StringConverter> sc)
+Script::Script(const std::shared_ptr<ContentManager>& content, const std::string& parent,
+    const std::string& name, std::string objName, std::unique_ptr<StringConverter> sc)
     : config(content->getContext()->getConfig())
     , database(content->getContext()->getDatabase())
     , content(content)
-    , runtime(runtime)
+    , runtime(content->getScriptingRuntime())
     , sc(std::move(sc))
-    , name(name)
+    , contextName(fmt::format("{}_{}", name, parent))
     , objectName(std::move(objName))
 {
     entrySeparator = config->getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
     /* create a context and associate it with the JS run time */
     ScriptingRuntime::AutoLock lock(runtime->getMutex());
-    ctx = runtime->createContext(name);
+    replaceAllString(contextName, "/", "_");
+    ctx = runtime->createContext(contextName);
     if (!ctx)
         throw_std_runtime_error("Scripting: could not initialize js context");
 
@@ -308,14 +311,35 @@ Script::Script(const std::shared_ptr<ContentManager>& content,
     }
     duk_put_prop_string(ctx, -2, autoscanItemPath.c_str()); // autoscan
 
+    for (auto&& bcs : ConfigDefinition::getConfigSetupList<ConfigBoxLayoutSetup>()) {
+        duk_push_object(ctx); // box-layout
+        std::string boxLayoutItemPath;
+        auto boxLayoutList = bcs->getValue()->getBoxLayoutListOption();
+        for (std::size_t i = 0; i < boxLayoutList->size(); i++) {
+            duk_push_object(ctx);
+            auto boxLayout = boxLayoutList->get(i);
+            setIntProperty("id", boxLayout->getId());
+            setIntProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_SIZE), boxLayout->getSize());
+            setIntProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_ENABLED), boxLayout->getEnabled());
+            setProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_TITLE), boxLayout->getTitle());
+            setProperty(ConfigDefinition::removeAttribute(ATTR_BOXLAYOUT_BOX_CLASS), boxLayout->getClass());
+            duk_put_prop_string(ctx, -2, boxLayout->getKey().c_str());
+        }
+        boxLayoutItemPath = bcs->getItemPath(ITEM_PATH_PREFIX); // prefix
+        duk_put_prop_string(ctx, -2, boxLayoutItemPath.c_str()); // box-layout
+    }
+
     duk_put_global_string(ctx, "config");
 
     defineFunctions(jsGlobalFunctions.data());
 
     std::string commonScrPath = config->getOption(CFG_IMPORT_SCRIPTING_COMMON_SCRIPT);
+    std::string commonFdrPath = config->getOption(CFG_IMPORT_SCRIPTING_COMMON_FOLDER);
 
-    if (commonScrPath.empty()) {
+    if (commonScrPath.empty() && commonFdrPath.empty()) {
         log_js("Common script disabled in configuration");
+    } else if (commonScrPath.empty()) {
+        loadFolder(commonFdrPath);
     } else {
         try {
             _load(commonScrPath);
@@ -325,6 +349,7 @@ Script::Script(const std::shared_ptr<ContentManager>& content,
         }
     }
     std::string customScrPath = config->getOption(CFG_IMPORT_SCRIPTING_CUSTOM_SCRIPT);
+    std::string customFdrPath = config->getOption(CFG_IMPORT_SCRIPTING_CUSTOM_FOLDER);
     if (!customScrPath.empty()) {
         try {
             _load(customScrPath);
@@ -332,12 +357,14 @@ Script::Script(const std::shared_ptr<ContentManager>& content,
         } catch (const std::runtime_error& e) {
             log_js("Unable to load {}: {}", customScrPath, e.what());
         }
+    } else if (!customFdrPath.empty()) {
+        loadFolder(customFdrPath);
     }
 }
 
 Script::~Script()
 {
-    runtime->destroyContext(name);
+    runtime->destroyContext(contextName);
 }
 
 Script* Script::getContextScript(duk_context* ctx)
@@ -369,7 +396,7 @@ void Script::defineFunctions(const duk_function_list_entry* functions)
 void Script::_load(const fs::path& scriptPath)
 {
     std::string scriptText = GrbFile(scriptPath).readTextFile();
-
+    this->scriptPath = scriptPath;
     if (scriptText.empty())
         throw_std_runtime_error("empty script");
 
@@ -399,22 +426,56 @@ void Script::load(const fs::path& scriptPath)
 void Script::_execute()
 {
     if (duk_pcall(ctx, 0) != DUK_EXEC_SUCCESS) {
-        log_error("Failed to execute script: {}", duk_safe_to_string(ctx, -1));
+#if DUK_VERSION > 20399
+        log_error("Failed to execute script {}: {}", scriptPath, duk_safe_to_stacktrace(ctx, -1));
+#else
+        log_error("Failed to execute script {}: {}", scriptPath, duk_safe_to_string(ctx, -1));
+#endif
         throw_std_runtime_error("Script: failed to execute script");
     }
     duk_pop(ctx);
 }
 
+void Script::loadFolder(const fs::path& scriptFolder)
+{
+    log_debug("Loading folder {}", scriptFolder.c_str());
+    std::error_code ec;
+    auto rootEntry = fs::directory_entry(scriptFolder);
+    if (!rootEntry.exists(ec) || !rootEntry.is_directory(ec)) {
+        log_error("Script folder not found: {}", scriptFolder.c_str());
+        return;
+    }
+    auto dirIterator = fs::directory_iterator(scriptFolder, ec);
+    if (ec) {
+        log_error("Failed to iterate {}, {}", scriptFolder.c_str(), ec.message());
+        return;
+    }
+    for (auto&& dirEntry : dirIterator) {
+        auto&& entryPath = dirEntry.path();
+        if (entryPath.extension() == ".js") {
+            try {
+                _load(entryPath);
+                _execute();
+                log_debug("Loaded {}", entryPath.c_str());
+            } catch (const std::runtime_error& e) {
+                log_error("Unable to load {}: {}", entryPath.c_str(), e.what());
+            }
+        }
+    }
+}
+
 #define GRB_CONTAINERTYPE_AUDIO "grb_container_type_audio"
 #define GRB_CONTAINERTYPE_IMAGE "grb_container_type_image"
 #define GRB_CONTAINERTYPE_VIDEO "grb_container_type_video"
 #define OBJECT_SCRIPT_PATH "object_script_path"
 #define OBJECT_AUTOSCAN_ID "object_autoscan_id"
+#define CONT_NAME "cont"
 
 void Script::cleanup()
 {
     duk_push_global_object(ctx);
     duk_del_prop_string(ctx, -1, objectName.c_str());
+    duk_del_prop_string(ctx, -1, CONT_NAME);
     duk_del_prop_string(ctx, -1, OBJECT_SCRIPT_PATH);
     duk_del_prop_string(ctx, -1, OBJECT_AUTOSCAN_ID);
     duk_del_prop_string(ctx, -1, GRB_CONTAINERTYPE_AUDIO);
@@ -427,6 +488,10 @@ void Script::execute(const std::shared_ptr<CdsObject>& obj, const std::string& r
     cdsObject2dukObject(obj);
     duk_put_global_string(ctx, objectName.c_str());
 
+    auto par = database->loadObject(obj->getParentID());
+    cdsObject2dukObject(par);
+    duk_put_global_string(ctx, CONT_NAME);
+
     duk_push_string(ctx, rootPath.c_str());
     duk_put_global_string(ctx, OBJECT_SCRIPT_PATH);
 
@@ -462,6 +527,65 @@ void Script::execute(const std::shared_ptr<CdsObject>& obj, const std::string& r
     duk_pop(ctx);
 }
 
+void Script::call(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const std::string& functionName, const fs::path& rootPath, const std::string& containerType)
+{
+    // write global object used in callback functionss
+    cdsObject2dukObject(obj);
+    duk_put_global_string(ctx, objectName.c_str());
+
+    // functionName(object, rootPath, autoScanId, containerType)
+
+    // Push function onto stack
+    if (!duk_get_global_string(ctx, functionName.c_str()) || !duk_is_function(ctx, -1)) {
+        log_error("javascript function not found: {}()", functionName);
+        duk_pop(ctx);
+        throw_std_runtime_error("javascript function not found: {}()", functionName);
+    }
+
+    int narg = 0;
+
+    // Push obj structure onto stack
+    cdsObject2dukObject(obj);
+    narg++;
+
+    // Push cont structure onto stack
+    auto par = cont ? cont : (obj->getParentID() >= CDS_ID_ROOT ? database->loadObject(obj->getParentID()) : nullptr);
+    cdsObject2dukObject(par ? par : obj);
+    narg++;
+
+    // push rootPath onto stack
+    duk_push_sprintf(ctx, "%s", rootPath.c_str());
+    narg++;
+
+    auto autoScan = content->getAutoscanDirectory(rootPath);
+
+    if (autoScan && !rootPath.empty()) {
+        // Push autoScanId onto stack
+        duk_push_sprintf(ctx, "%d", autoScan->getScanID());
+        narg++;
+    } else {
+        // Push autoScanId onto stack
+        duk_push_sprintf(ctx, "%d", -1);
+        narg++;
+    }
+    // Container Type onto stack
+    duk_push_sprintf(ctx, "%s", containerType.c_str());
+    narg++;
+
+    if (duk_pcall(ctx, (duk_idx_t)narg) != DUK_EXEC_SUCCESS) {
+        // Note: The invoked function will be blamed for execution errors, not the actual offending line of code
+        // https://github.com/svaarala/duktape/blob/master/doc/error-objects.rst
+#if DUK_VERSION > 20399
+        log_error("javascript runtime error: {}() - {}\n", functionName, duk_safe_to_stacktrace(ctx, -1));
+#else
+        log_error("javascript runtime error: {}() - {}\n", functionName, duk_safe_to_string(ctx, -1));
+#endif
+        duk_pop(ctx);
+        throw_std_runtime_error("javascript runtime error");
+    }
+    duk_pop(ctx);
+}
+
 void Script::setMetaData(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsItem>& item, const std::string& sym, const std::string& val) const
 {
     if (sym == MetadataHandler::getMetaFieldName(M_TRACKNUMBER) && item) {
@@ -540,16 +664,18 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
                     auto purpSym = fmt::format("{}:purpose", resCount);
                     int purpose = getIntProperty(purpSym, -1);
                     if (ht >= 0 && purpose >= 0) {
-                        obj->addResource(std::make_shared<CdsResource>(MetadataHandler::remapContentHandler(ht), CdsResource::remapPurpose(purpose)));
+                        auto newRes = std::make_shared<CdsResource>(MetadataHandler::remapContentHandler(ht), CdsResource::remapPurpose(purpose));
+                        obj->addResource(newRes);
+                        newRes->setResId(resCount);
                     }
                     resCount++;
                 }
             }
-            resCount = 0;
             for (auto&& res : obj->getResources()) {
-                // only attribute enumerated in res_keys is allowed
+                resCount = res->getResId();
+                // only attributes enumerated in res_names are allowed
                 for (auto&& [key, upnp] : res_names) {
-                    auto val = getProperty(resCount == 0 ? upnp.data() : fmt::format("{}-{}", resCount, upnp));
+                    auto val = getProperty(resCount == 0 ? CdsResource::getAttributeName(key) : fmt::format("{}-{}", resCount, CdsResource::getAttributeName(key)));
                     if (!val.empty()) {
                         val = sc->convert(val);
                         res->addAttribute(key, val);
@@ -571,7 +697,6 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
                         res->addOption(key, val);
                     }
                 }
-                resCount++;
             }
         }
         duk_pop(ctx); // res
@@ -642,6 +767,19 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
     }
     duk_pop(ctx); // metaData
 
+    fs::path location = getProperty("location");
+    if (!location.empty()) {
+        // location must not be touched by character conversion!
+        obj->setLocation(location);
+    }
+
+    auto description = getProperty("description");
+    if (!description.empty()) {
+        description = sc->convert(description);
+        obj->removeMetaData(M_DESCRIPTION);
+        obj->addMetaData(M_DESCRIPTION, description);
+    }
+
     // CdsItem
     if (obj->isItem()) {
         auto item = std::static_pointer_cast<CdsItem>(obj);
@@ -650,57 +788,39 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         if (pcd)
             pcdItem = std::static_pointer_cast<CdsItem>(pcd);
 
-        {
-            auto val = getProperty("mimetype");
-            if (!val.empty()) {
-                val = sc->convert(val);
-                item->setMimeType(val);
-            } else if (pcdItem) {
-                item->setMimeType(pcdItem->getMimeType());
-            }
+        auto mimetype = getProperty("mimetype");
+        if (!mimetype.empty()) {
+            mimetype = sc->convert(mimetype);
+            item->setMimeType(mimetype);
+        } else if (pcdItem) {
+            item->setMimeType(pcdItem->getMimeType());
         }
 
-        {
-            auto val = getProperty("serviceID");
-            if (!val.empty()) {
-                val = sc->convert(val);
-                item->setServiceID(val);
-            }
+        auto serviceID = getProperty("serviceID");
+        if (!serviceID.empty()) {
+            serviceID = sc->convert(serviceID);
+            item->setServiceID(serviceID);
         }
 
-        {
-            auto val = getProperty("description");
-            if (!val.empty()) {
-                val = sc->convert(val);
-                item->removeMetaData(M_DESCRIPTION);
-                item->addMetaData(M_DESCRIPTION, val);
-            } else if (pcdItem && item->getMetaData(M_DESCRIPTION).empty() && !pcdItem->getMetaData(M_DESCRIPTION).empty()) {
-                item->addMetaData(M_DESCRIPTION, pcdItem->getMetaData(M_DESCRIPTION));
-            }
+        if (description.empty() && pcd && obj->getMetaData(M_DESCRIPTION).empty() && !obj->getMetaData(M_DESCRIPTION).empty()) {
+            obj->addMetaData(M_DESCRIPTION, obj->getMetaData(M_DESCRIPTION));
         }
-        handleObject2cdsItem(ctx, pcd, item);
 
-        // location must not be touched by character conversion!
-        fs::path location = getProperty("location");
-        if (!location.empty())
-            obj->setLocation(location);
-        else {
-            if (pcd)
-                obj->setLocation(pcd->getLocation());
+        if (location.empty() && pcd) {
+            obj->setLocation(pcd->getLocation());
         }
 
         if (obj->isExternalItem()) {
             std::string protocolInfo;
 
             obj->setRestricted(true);
-            {
-                auto val = getProperty("protocol");
-                if (!val.empty()) {
-                    val = sc->convert(val);
-                    protocolInfo = renderProtocolInfo(item->getMimeType(), val);
-                } else {
-                    protocolInfo = renderProtocolInfo(item->getMimeType(), PROTOCOL);
-                }
+
+            auto protocol = getProperty("protocol");
+            if (!protocol.empty()) {
+                protocol = sc->convert(protocol);
+                protocolInfo = renderProtocolInfo(item->getMimeType(), protocol);
+            } else {
+                protocolInfo = renderProtocolInfo(item->getMimeType(), PROTOCOL);
             }
 
             std::shared_ptr<CdsResource> resource;
@@ -716,19 +836,22 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
                 resource->addAttribute(CdsResource::Attribute::SIZE, fmt::to_string(size));
             }
         }
+
+        handleObject2cdsItem(ctx, pcd, item);
     }
 
     // CdsDirectory
     if (obj->isContainer()) {
         auto cont = std::static_pointer_cast<CdsContainer>(obj);
-        handleObject2cdsContainer(ctx, pcd, cont);
         auto id = getIntProperty("updateID", -1);
-        if (id >= 0)
+        if (id >= CDS_ID_ROOT)
             cont->setUpdateID(id);
 
         int searchable = getBoolProperty("searchable");
         if (searchable >= 0)
             cont->setSearchable(searchable);
+
+        handleObject2cdsContainer(ctx, pcd, cont);
     }
 
     return obj;
@@ -854,15 +977,15 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
     {
         duk_push_object(ctx);
         // stack: js res_js
-
+        setProperty("count", fmt::to_string(obj->getResourceCount()));
         if (obj->getResourceCount() > 0) {
-            int resCount = 0;
+            std::size_t resCount = 0;
             for (auto&& res : obj->getResources()) {
                 setProperty(fmt::format("{}:handlerType", resCount), fmt::to_string(to_underlying(res->getHandlerType())));
                 setProperty(fmt::format("{}:purpose", resCount), fmt::to_string(to_underlying(res->getPurpose())));
                 auto attributes = res->getAttributes();
                 for (auto&& [key, attr] : attributes) {
-                    setProperty(resCount == 0 ? CdsResource::getAttributeName(key) : fmt::format("{}-{}", resCount, key), attr);
+                    setProperty(resCount == 0 ? CdsResource::getAttributeName(key) : fmt::format("{}-{}", resCount, CdsResource::getAttributeName(key)), attr);
                 }
                 auto parameters = res->getParameters();
                 for (auto&& [key, param] : parameters) {
