@@ -55,7 +55,7 @@ protected:
 private:
     std::unique_ptr<Quirks> getQuirks(const UpnpFileInfo* info) const;
     static std::size_t parseResourceInfo(std::map<std::string, std::string>& params);
-    std::unique_ptr<MetadataHandler> getResourceMetadataHandler(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource) const;
+    std::unique_ptr<MetadataHandler> getResourceMetadataHandler(std::shared_ptr<CdsObject>& obj, std::shared_ptr<CdsResource>& resource) const;
 };
 
 #endif // __FILE_REQUEST_HANDLER_H__
