@@ -119,6 +119,8 @@ void ContentDirectoryService::doBrowse(ActionRequest& request)
     try {
         if (arr.empty())
             arr = database->browse(param);
+        else
+            param.setTotalMatches(arr.size());
     } catch (const std::runtime_error& e) {
         log_error("No such object: {}", e.what());
         throw UpnpException(UPNP_E_NO_SUCH_ID, "no such object");
@@ -308,6 +310,35 @@ void ContentDirectoryService::doGetSortCapabilities(ActionRequest& request) cons
     log_debug("end");
 }
 
+void ContentDirectoryService::doGetFeatureList(ActionRequest& request) const
+{
+    log_debug("start");
+
+    auto response = xmlBuilder->createResponse(request.getActionName(), UPNP_DESC_CDS_SERVICE_TYPE);
+    auto root = response->document_element();
+    pugi::xml_document respRoot;
+    auto features = respRoot.append_child("Features");
+    features.append_attribute("xmlns") = "urn:schemas-upnp-org:av:avs";
+    features.append_attribute("xmlns:xsi") = "http://www.w3.org/2001/XMLSchema-instance";
+    features.append_attribute("xsi:schemaLocation") = "urn:schemas-upnp-org:av:avs http://www.upnp.org/schemas/av/avs.xsd";
+    root.append_child("FeatureList").append_child(pugi::node_pcdata).set_value(UpnpXMLBuilder::printXml(respRoot, "", 0).c_str());
+    request.setResponse(std::move(response));
+
+    log_debug("end");
+}
+
+void ContentDirectoryService::doGetSortExtensionCapabilities(ActionRequest& request) const
+{
+    log_debug("start");
+
+    auto response = xmlBuilder->createResponse(request.getActionName(), UPNP_DESC_CDS_SERVICE_TYPE);
+    auto root = response->document_element();
+    root.append_child("SortExtensionCaps").append_child(pugi::node_pcdata).set_value("+,-"); // TIME+, TIME-
+    request.setResponse(std::move(response));
+
+    log_debug("end");
+}
+
 void ContentDirectoryService::doGetSystemUpdateID(ActionRequest& request) const
 {
     log_debug("start");
@@ -368,6 +399,10 @@ void ContentDirectoryService::processActionRequest(ActionRequest& request)
         doGetSortCapabilities(request);
     } else if (request.getActionName() == "GetSystemUpdateID") {
         doGetSystemUpdateID(request);
+    } else if (request.getActionName() == "GetFeatureList") {
+        doGetFeatureList(request);
+    } else if (request.getActionName() == "GetSortExtensionCapabilities") {
+        doGetSortExtensionCapabilities(request);
     } else if (request.getActionName() == "Search") {
         doSearch(request);
     } else if (request.getActionName() == "X_SetBookmark") {
