@@ -31,11 +31,16 @@
 
 #include "config/client_config.h"
 #include "config/config_definition.h"
+#include "config/config_option_enum.h"
 #include "config/config_setup.h"
 #include "config/directory_tweak.h"
 #include "config/dynamic_content.h"
+#include "config/setup/config_setup_autoscan.h"
+#include "config/setup/config_setup_boxlayout.h"
+#include "config/setup/config_setup_enum.h"
 #include "content/autoscan.h"
 #include "content/content_manager.h"
+#include "content/layout/box_layout.h"
 #include "database/database.h"
 #include "metadata/metadata_handler.h"
 #include "transcoding/transcoding.h"
@@ -408,6 +413,49 @@ void Web::ConfigLoad::process()
         createItem(item, cs->getItemPath(ITEM_PATH_NEW, ATTR_DYNAMIC_CONTAINER_MAXCOUNT), cs->option, ATTR_DYNAMIC_CONTAINER_MAXCOUNT, ConfigDefinition::findConfigSetup(ATTR_DYNAMIC_CONTAINER_MAXCOUNT));
     }
 
+    // write box layout content
+    cs = ConfigDefinition::findConfigSetup(CFG_BOXLAYOUT_BOX);
+    auto boxlayoutContent = cs->getValue()->getBoxLayoutListOption();
+    for (std::size_t i = 0; i < boxlayoutContent->size(); i++) {
+        auto cont = boxlayoutContent->get(i);
+
+        auto item = values.append_child("item");
+        createItem(item, cs->getItemPath(i, ATTR_BOXLAYOUT_BOX_KEY), cs->option, ATTR_BOXLAYOUT_BOX_KEY);
+        setValue(item, cont->getKey());
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(i, ATTR_BOXLAYOUT_BOX_TITLE), cs->option, ATTR_BOXLAYOUT_BOX_TITLE);
+        setValue(item, cont->getTitle());
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(i, ATTR_BOXLAYOUT_BOX_CLASS), cs->option, ATTR_BOXLAYOUT_BOX_CLASS);
+        setValue(item, cont->getClass());
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(i, ATTR_BOXLAYOUT_BOX_SIZE), cs->option, ATTR_BOXLAYOUT_BOX_SIZE);
+        setValue(item, cont->getSize());
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(i, ATTR_BOXLAYOUT_BOX_ENABLED), cs->option, ATTR_BOXLAYOUT_BOX_ENABLED);
+        setValue(item, cont->getEnabled());
+    }
+    if (boxlayoutContent->size() == 0) {
+        auto item = values.append_child("item");
+        createItem(item, cs->getItemPath(ITEM_PATH_NEW, ATTR_BOXLAYOUT_BOX_KEY), cs->option, ATTR_BOXLAYOUT_BOX_KEY, ConfigDefinition::findConfigSetup(ATTR_BOXLAYOUT_BOX_KEY));
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(ITEM_PATH_NEW, ATTR_BOXLAYOUT_BOX_TITLE), cs->option, ATTR_BOXLAYOUT_BOX_TITLE, ConfigDefinition::findConfigSetup(ATTR_BOXLAYOUT_BOX_TITLE));
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(ITEM_PATH_NEW, ATTR_BOXLAYOUT_BOX_CLASS), cs->option, ATTR_BOXLAYOUT_BOX_CLASS, ConfigDefinition::findConfigSetup(ATTR_BOXLAYOUT_BOX_CLASS));
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(ITEM_PATH_NEW, ATTR_BOXLAYOUT_BOX_SIZE), cs->option, ATTR_BOXLAYOUT_BOX_SIZE, ConfigDefinition::findConfigSetup(ATTR_BOXLAYOUT_BOX_SIZE));
+
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(ITEM_PATH_NEW, ATTR_BOXLAYOUT_BOX_ENABLED), cs->option, ATTR_BOXLAYOUT_BOX_ENABLED, ConfigDefinition::findConfigSetup(ATTR_BOXLAYOUT_BOX_ENABLED));
+    }
+
     // write transconding configuration
     cs = ConfigDefinition::findConfigSetup(CFG_TRANSCODING_PROFILE_LIST);
     auto transcoding = cs->getValue()->getTranscodingProfileListOption();
@@ -426,6 +474,10 @@ void Web::ConfigLoad::process()
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_FILTER, ATTR_TRANSCODING_PROFILES_PROFLE_SRCDLNA), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_SRCDLNA, cs);
         setValue(item, filter->getSourceProfile());
 
+        item = values.append_child("item");
+        createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_FILTER, ATTR_TRANSCODING_PROFILES_PROFLE_NOTRANSCODING), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_NOTRANSCODING, cs);
+        setValue(item, fmt::join(filter->getNoTranscodingMimeTypes(), ","));
+
         item = values.append_child("item");
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_FILTER, ATTR_TRANSCODING_PROFILES_PROFLE_CLIENTFLAGS), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_CLIENTFLAGS, cs);
         setValue(item, ClientConfig::mapFlags(filter->getClientFlags()));
@@ -451,8 +503,9 @@ void Web::ConfigLoad::process()
         setValue(item, entry->isEnabled());
 
         item = values.append_child("item");
+        auto ttEnumSetup = ConfigDefinition::findConfigSetup<ConfigEnumSetup<TranscodingType>>(ATTR_TRANSCODING_PROFILES_PROFLE_TYPE);
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_TYPE), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_TYPE);
-        setValue(item, (entry->getType() == TR_External ? "external" : "none"));
+        setValue(item, ttEnumSetup->mapEnumValue(entry->getType()));
 
         item = values.append_child("item");
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_DLNAPROF), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_DLNAPROF);
@@ -515,10 +568,11 @@ void Web::ConfigLoad::process()
         setValue(item, entry->getBufferInitialFillSize());
 
         auto fourCCMode = entry->getAVIFourCCListMode();
-        if (fourCCMode != FCC_None) {
+        if (fourCCMode != AviFourccListmode::None) {
             item = values.append_child("item");
             createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE);
-            setValue(item, TranscodingProfile::mapFourCcMode(fourCCMode));
+            auto fccEnumSetup = ConfigDefinition::findConfigSetup<ConfigEnumSetup<AviFourccListmode>>(ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_MODE);
+            setValue(item, fccEnumSetup->mapEnumValue(fourCCMode));
 
             const auto& fourCCList = entry->getAVIFourCCList();
             if (!fourCCList.empty()) {
@@ -535,7 +589,7 @@ void Web::ConfigLoad::process()
         auto autoscan = ascs->getValue()->getAutoscanListOption();
         for (std::size_t i = 0; i < autoscan.size(); i++) {
             auto&& entry = autoscan.at(i);
-            auto&& adir = content->getAutoscanDirectory(entry.getLocation());
+            auto&& adir = content->getAutoscanDirectory(entry->getLocation());
             auto item = values.append_child("item");
             createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_LOCATION), ascs->option, ATTR_AUTOSCAN_DIRECTORY_LOCATION);
             setValue(item, adir->getLocation());
@@ -546,7 +600,7 @@ void Web::ConfigLoad::process()
 
             item = values.append_child("item");
             createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_INTERVAL), ascs->option, ATTR_AUTOSCAN_DIRECTORY_INTERVAL);
-            setValue(item, adir->getInterval());
+            setValue(item, adir->getInterval().count());
 
             item = values.append_child("item");
             createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_RECURSIVE), ascs->option, ATTR_AUTOSCAN_DIRECTORY_RECURSIVE);
@@ -554,12 +608,16 @@ void Web::ConfigLoad::process()
 
             item = values.append_child("item");
             createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_MEDIATYPE), ascs->option, ATTR_AUTOSCAN_DIRECTORY_MEDIATYPE);
-            setValue(item, adir->getMediaType());
+            setValue(item, AutoscanDirectory::mapMediaType(adir->getMediaType()));
 
             item = values.append_child("item");
             createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES), ascs->option, ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES);
             setValue(item, adir->getHidden());
 
+            item = values.append_child("item");
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_FOLLOWSYMLINKS), ascs->option, ATTR_AUTOSCAN_DIRECTORY_FOLLOWSYMLINKS);
+            setValue(item, adir->getFollowSymlinks());
+
             item = values.append_child("item");
             createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_CONTAINER_TYPE_AUDIO), ascs->option, ATTR_AUTOSCAN_CONTAINER_TYPE_AUDIO);
             setValue(item, adir->getContainerTypes().at(AutoscanMediaMode::Audio));
@@ -641,14 +699,22 @@ void Web::ConfigLoad::process()
     // write content of all vectors
     for (auto&& vcs : ConfigDefinition::getConfigSetupList<ConfigVectorSetup>()) {
         int i = 0;
-        auto vector = vcs->getValue()->getVectorOption(true);
-        for (auto&& value : vector) {
-            for (auto&& [key, val] : value) {
+        auto vectorOption = vcs->getValue()->getVectorOption(true);
+        std::vector<std::string> attrList;
+        for (auto& opt : vcs->optionList) {
+            attrList.push_back(ConfigDefinition::removeAttribute(opt));
+        }
+        for (auto&& vector : vectorOption) {
+            for (auto&& [key, val] : vector) {
                 auto item = values.append_child("item");
-                createItem(item, vcs->getItemPath(i, vcs->option) + key, vcs->option, vcs->optionList[0], vcs); // FIxME
+                int j = 0;
+                auto pos = std::find(attrList.begin(), attrList.end(), key);
+                if (pos != attrList.end())
+                    j = std::distance(attrList.begin(), pos);
+                createItem(item, vcs->getItemPath(i, key), vcs->option, vcs->optionList[j], vcs);
                 setValue(item, val);
-                i++;
             }
+            i++;
         }
     }
 
